<?php

namespace Drupal\charts_highcharts_maps\Plugin\views\style;

use Drupal\charts\Plugin\chart\Library\ChartInterface;
use Drupal\charts\Plugin\views\style\ChartsPluginStyleChart;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Html;
use Drupal\core\form\FormStateInterface;
use Drupal\Core\Render\Element;

/**
 * Style plugin to render view as a chart.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "chart_highmap",
 *   title = @Translation("HighMaps"),
 *   help = @Translation("Render a map of your data."),
 *   theme = "views_view_charts_highcharts_maps",
 *   display_types = { "normal" }
 * )
 */
class HighMapsPluginStyleChart extends ChartsPluginStyleChart {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['topology']['default'] = '';
    $options['subtitle']['default'] = $this->t("Source map: <a href='https://code.highcharts.com/mapdata/custom/world.geo.json'>World Map</a>");

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $data_options = $this->displayHandler->getFieldLabels();
    $data_options[''] = $this->t('None');

    $form['topology'] = [
      '#type' => 'select',
      '#title' => t('Map Field'),
      '#options' => $data_options,
      '#default_value' => $this->options['topology'] ?? '',
    ];

    $form['subtitle'] = [
      '#type' => 'textfield',
      '#title' => t('Map Subtitle'),
      '#size' => 70,
      '#maxlength' => 250,
      '#description' => $this->t('Leave blank for no subtitle.'),
      '#default_value' => $this->options['subtitle'] ?? '',
    ];

    $form['display']['background_color'] = [
      '#title' => $this->t('Legend background color'),
      '#type' => 'textfield',
      '#size' => 10,
      '#maxlength' => 7,
      '#attributes' => ['placeholder' => $this->t('transparent')],
      '#description' => $this->t('Leave blank for a transparent background.'),
     // '#default_value' => $legend_configuration['background_color'] ?? '',
    ];

    $form_state->set('default_options', $this->options);
  }

  /**
   * Returns the key of the Label Field.
   *
   * @return string
   *   The Label Field key.
   */
  private function getLabelFieldKey() {
    if (!isset($this->labelFieldKey)) {
      $field_handlers = $this->view->getHandlers('field');
      $chart_settings = $this->options['chart_settings'];
      $chart_fields = $chart_settings['fields'];
      $label_field = $field_handlers[$chart_fields['label']] ?? '';
      $this->labelFieldKey = $label_field ? $chart_fields['label'] : '';
    }

    return $this->labelFieldKey;
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $chart = parent::render();

    // Get GeoJson data for the country.
    $map_field =  $this->options['topology'];
    $first_row = $this->view->result[0];
    $country = $first_row->_relationship_entities['field_country']->get($map_field)->getValue();
    $chart['#topology'] = json_decode($country[0]['value']);

    $chart['#subtitle'] = $this->options['subtitle'];

    return $chart;
  }

}
